;; Copyright (c) 2006 Carlos Ungil

;; Permission is hereby granted, free of charge, to any person obtaining
;; a copy of this software and associated documentation files (the
;; "Software"), to deal in the Software without restriction, including
;; without limitation the rights to use, copy, modify, merge, publish,
;; distribute, sublicense, and/or sell copies of the Software, and to
;; permit persons to whom the Software is furnished to do so, subject to
;; the following conditions:

;; The above copyright notice and this permission notice shall be
;; included in all copies or substantial portions of the Software.

;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
;; NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
;; LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
;; OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
;; WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(in-package :rcl)

(defvar *r-output* t)
(defvar *r-message* t)

(defvar *r-output-prefix* "R# ")
(defvar *r-message-prefix* "R! ")

(defmacro with-r-output ((&optional stream prefix) &body body)
  `(let ((sink (r-funcall "textConnection" "tmpout" "w"))
	 result)
     (r-funcall "sink" :file sink :type "output")
     (unwind-protect
	  (setf result (progn ,@body))
       (r-funcall "sink" :type "output")
       (r-funcall "close.connection" sink)
       (format ,(or stream '*r-output*) 
	       (concatenate 'string "~{"  ,(or prefix '*r-output-prefix*) "~A~&~}")
	       (let ((*extract-single-element* nil))
		 (r-obj-decode (r-variable "tmpout")))))
     result))
  
(defmacro with-r-message ((&optional (stream *r-message*) (prefix *r-message-prefix*)) &body body)
  `(let ((sink (r-funcall "textConnection" "tmperr" "w"))
	 result)
     (r-funcall "sink" :file sink :type "message")
     (unwind-protect
	  (setf result (progn ,@body))
       (r-funcall "sink" :type "message")
       (r-funcall "close.connection" sink)
       (format ,stream (concatenate 'string "~{"  ,prefix "~A~&~}")
	       (let ((*extract-single-element* nil))
		 (r-obj-decode (r-variable "tmperr")))))
     result))

  


